# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from barbicanclient import client as barbicanclient
from glanceclient import client as glanceclient
from heatclient import client as heatclient
from unittest import mock

from keystoneclient.auth.identity import v2 as identity_v2
from keystoneclient import exceptions
from keystoneclient import session
from mistralclient.api import client as mistralclient
from neutronclient.neutron import client as neutronclient
from oslo_config import cfg
from swiftclient import client as swiftclient
from zaqarclient.queues.v1 import client as zaqarclient

from solum.common import clients
from solum.common import exception
from solum.tests import base


class ClientsTest(base.BaseTestCase):

    @mock.patch.object(clients.OpenStackClients, 'keystone')
    def test_url_for(self, mock_keystone):
        obj = clients.OpenStackClients(None)
        obj.url_for(service_type='fake_service', interface='fake_endpoint',
                    region_name='FakeRegion')

        mock_cat = mock_keystone.return_value.client.service_catalog
        mock_cat.url_for.assert_called_once_with(service_type='fake_service',
                                                 interface='fake_endpoint',
                                                 region_name='FakeRegion')

    @mock.patch.object(barbicanclient, 'Client')
    @mock.patch.object(session, 'Session')
    def test_clients_barbican(self, mock_sess, mock_call):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        mock_sess.return_value = "keystone_session"
        mock_call.return_value = "barbican_client_handle"
        obj = clients.OpenStackClients(None)
        self.assertIsNone(obj._barbican)
        obj.barbican().admin_client
        self.assertIsNotNone(obj._barbican)
        mock_call.assert_called_once_with(session='keystone_session')

    def test_clients_barbican_noauth(self):
        self.skipTest('Skipping barbican noauth test.')
        dummy_url = 'http://server.test:5000/v2.0'
        cfg.CONF.set_override('www_authenticate_uri', dummy_url,
                              group='keystone_authtoken')
        cfg.CONF.set_override('username', 'solum',
                              group='keystone_authtoken')
        cfg.CONF.set_override('password', 'verybadpass',
                              group='keystone_authtoken')
        cfg.CONF.set_override('project_name', 'service',
                              group='keystone_authtoken')
        obj = clients.OpenStackClients(None)

        # try to create and store a secret
        try:
            bclient = obj.barbican().admin_client
            secret = bclient.secrets.create(name="test", payload="test")
            secret.store()
        except exceptions.ConnectionRefused:
            self.assertTrue(True)
        except exceptions.RequestTimeout:
            self.assertTrue(True)

    @mock.patch.object(barbicanclient, 'Client')
    @mock.patch.object(identity_v2, 'Password')
    @mock.patch.object(session, 'Session')
    def test_clients_barbican_cached(self, mock_sess, mock_auth, mock_call):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        mock_auth.return_value = "keystone_auth_handle"
        mock_call.return_value = "barbican_client_handle"
        mock_sess.return_value = "keystone_session"
        obj = clients.OpenStackClients(None)
        barbican_admin = obj.barbican().admin_client
        barbican_admin_cached = obj.barbican().admin_client
        self.assertEqual(barbican_admin, barbican_admin_cached)
        mock_call.assert_called_once_with(session='keystone_session')

    @mock.patch.object(glanceclient, 'Client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    def test_clients_glance(self, mock_url, mock_call):
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._glance = None
        obj.glance()
        mock_call.assert_called_once_with(
            '2', 'url_from_keystone', token='3bcc3d3a03f44e3d8377f9247b0ad155')
        mock_url.assert_called_once_with(service_type='image',
                                         interface='publicURL',
                                         region_name='RegionOne')

    def test_clients_glance_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        obj = clients.OpenStackClients(con)
        obj._glance = None
        self.assertRaises(exception.AuthorizationFailure, obj.glance)

    @mock.patch.object(glanceclient, 'Client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    def test_clients_glance_cached(self, mock_url, mock_call):
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._glance = None
        glance = obj.glance()
        glance_cached = obj.glance()
        self.assertEqual(glance, glance_cached)

    @mock.patch.object(heatclient, 'Client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_heat(self, mock_auth, mock_url, mock_call):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        con.auth_url = "keystone_url"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._heat = None
        obj.heat()
        mock_call.assert_called_once_with(
            '1', 'url_from_keystone', username=None,
            cert_file=None, token='3bcc3d3a03f44e3d8377f9247b0ad155',
            auth_url='keystone_url', ca_file=None, key_file=None,
            password=None, insecure=False)
        mock_url.assert_called_once_with(service_type='orchestration',
                                         interface='publicURL',
                                         region_name='RegionOne')

    def test_clients_heat_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        auth_url = mock.PropertyMock(name="auth_url",
                                     return_value="keystone_url")
        type(con).auth_url = auth_url
        con.get_url_for = mock.Mock(name="get_url_for")
        con.get_url_for.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._heat = None
        self.assertRaises(exception.AuthorizationFailure, obj.heat)

    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_heat_cached(self, mock_auth, mock_url):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        con.auth_url = "keystone_url"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._heat = None
        heat = obj.heat()
        heat_cached = obj.heat()
        self.assertEqual(heat, heat_cached)

    @mock.patch.object(swiftclient, 'Connection')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    def test_clients_swift(self, mock_url, mock_call):
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._swift = None
        obj.swift()
        mock_call.assert_called_once_with(
            auth_version="2.0",
            os_options={'endpoint_type': 'publicURL',
                        'region_name': 'RegionOne'},
            cacert=None, preauthurl="url_from_keystone", insecure=False,
            preauthtoken="3bcc3d3a03f44e3d8377f9247b0ad155")

    def test_clients_swift_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        auth_url = mock.PropertyMock(name="auth_url",
                                     return_value="keystone_url")
        type(con).auth_url = auth_url
        con.get_url_for = mock.Mock(name="get_url_for")
        con.get_url_for.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._swift = None
        self.assertRaises(exception.AuthorizationFailure, obj.swift)

    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_swift_not_cached(self, mock_auth, mock_url):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._swift = None
        swift = obj.swift()
        swift_cached = obj.swift()
        self.assertNotEqual(swift, swift_cached)

    @mock.patch.object(neutronclient, 'Client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_neutron(self, mock_auth, mock_url, mock_call):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        con.auth_url = "keystone_url"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._neutron = None
        obj.neutron()
        mock_call.assert_called_once_with(
            '2.0', endpoint_url='url_from_keystone', username=None,
            token='3bcc3d3a03f44e3d8377f9247b0ad155', auth_url='keystone_url',
            ca_cert=None, password=None, insecure=False)

    def test_clients_neutron_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        auth_url = mock.PropertyMock(name="auth_url",
                                     return_value="keystone_url")
        type(con).auth_url = auth_url
        con.get_url_for = mock.Mock(name="get_url_for")
        con.get_url_for.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._neutron = None
        self.assertRaises(exception.AuthorizationFailure, obj.neutron)

    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_neutron_cached(self, mock_auth, mock_url):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._neutron = None
        neutron = obj.neutron()
        neutron_cached = obj.neutron()
        self.assertEqual(neutron, neutron_cached)

    @mock.patch.object(mistralclient, 'client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    def test_clients_mistral(self, mock_url, mock_call):
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._mistral = None
        obj.mistral()
        mock_call.assert_called_once_with(
            mistral_url='url_from_keystone',
            auth_token='3bcc3d3a03f44e3d8377f9247b0ad155')
        mock_url.assert_called_once_with(service_type='workflow',
                                         interface='publicURL',
                                         region_name='RegionOne')

    def test_clients_mistral_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        obj = clients.OpenStackClients(con)
        obj._mistral = None
        self.assertRaises(exception.AuthorizationFailure, obj.mistral)

    @mock.patch.object(mistralclient, 'client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    def test_clients_mistral_cached(self, mock_url, mock_call):
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54d"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._mistral = None
        mistral = obj.mistral()
        mistral_cached = obj.mistral()
        self.assertEqual(mistral, mistral_cached)

    @mock.patch.object(zaqarclient, 'Client')
    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_zaqar(self, mock_auth, mock_url, mock_call):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        con.auth_url = "keystone_url"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._zaqar = None
        obj.zaqar()
        conf = {'auth_opts':
                {'backend': 'keystone',
                 'options':
                    {'os_auth_token': '3bcc3d3a03f44e3d8377f9247b0ad155',
                     'os_auth_url': 'keystone_url',
                     'insecure': False}
                 }
                }
        mock_call.assert_called_once_with('url_from_keystone', conf=conf)

    def test_clients_zaqar_noauth(self):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        con = mock.MagicMock()
        con.auth_token = None
        con.auth_token_info = None
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        auth_url = mock.PropertyMock(name="auth_url",
                                     return_value="keystone_url")
        type(con).auth_url = auth_url
        con.get_url_for = mock.Mock(name="get_url_for")
        con.get_url_for.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._zaqar = None
        self.assertRaises(exception.AuthorizationFailure, obj.zaqar)

    @mock.patch.object(clients.OpenStackClients, 'url_for')
    @mock.patch.object(clients.OpenStackClients, 'auth_url')
    def test_clients_zaqar_cached(self, mock_auth, mock_url):
        mock_auth.__get__ = mock.Mock(return_value="keystone_url")
        con = mock.MagicMock()
        con.tenant = "b363706f891f48019483f8bd6503c54b"
        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
        mock_url.return_value = "url_from_keystone"
        obj = clients.OpenStackClients(con)
        obj._zaqar = None
        zaqar = obj.zaqar()
        zaqar_cached = obj.zaqar()
        self.assertEqual(zaqar, zaqar_cached)



# Copyright (c) 2016 Hewlett Packard Enterprise Development Corporation, LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_middleware import cors
from oslo_policy import opts


def set_config_defaults():
    """This method updates all configuration default values."""
    set_cors_middleware_defaults()

    # TODO(gmann): Remove setting the default value of config policy_file
    # once oslo_policy change the default value to 'policy.yaml'.
    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
    opts.set_defaults(cfg.CONF, 'policy.yaml')


def set_cors_middleware_defaults():
    """Update default configuration options for oslo.middleware."""
    # CORS Defaults
    # TODO(krotscheck): Update with https://review.opendev.org/#/c/285368/
    cfg.set_defaults(cors.CORS_OPTS,
                     allow_headers=['X-Auth-Token',
                                    'X-Openstack-Request-Id',
                                    'X-Subject-Token'],
                     expose_headers=['X-Auth-Token',
                                     'X-Openstack-Request-Id',
                                     'X-Subject-Token'],
                     allow_methods=['GET',
                                    'PUT',
                                    'POST',
                                    'DELETE',
                                    'PATCH']
                     )



# Copyright 2013 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils

from solum.api.handlers import handler
from solum.common import exception
from solum.common import keystone_utils
from solum.common import repo_utils
from solum.conductor import api as conductor_api
from solum.deployer import api as deploy_api
from solum import objects
from solum.objects import assembly
from solum.objects import image
from solum.worker import api as worker_api


LOG = logging.getLogger(__name__)

# Register options for the service
API_SERVICE_OPTS = [
    cfg.StrOpt('image_format',
               default='qcow2',
               help='The format of the image to output'),
    cfg.StrOpt('source_format',
               default='heroku',
               help='The format of source repository'),
    cfg.StrOpt('rebuild_phrase',
               default='solum retry tests',
               help='Comment phrase to trigger rebuilding'),
]


def list_opts():
    yield 'api', API_SERVICE_OPTS


CONF = cfg.CONF
CONF.register_opts(API_SERVICE_OPTS, group='api')

ASSEMBLY_STATES = assembly.States
IMAGE_STATES = image.States


class AssemblyHandler(handler.Handler):
    """Fulfills a request on the assembly resource."""

    def get(self, id):
        """Return an assembly."""
        return objects.registry.Assembly.get_by_uuid(self.context, id)

    def trigger_workflow(self, trigger_id, commit_sha='',
                         status_url=None, collab_url=None):
        """Get trigger by trigger id and start git workflow associated."""
        # Note: self.context will be None at this point as this is a
        # non-authenticated request.
        db_obj = objects.registry.Assembly.get_by_trigger_id(None,
                                                             trigger_id)
        try:
            # get the trust\impersonation context and authenticate it.
            self.context = keystone_utils.create_delegation_context(
                db_obj, self.context)
        except exception.AuthorizationFailure as auth_ex:
            LOG.warning(auth_ex)
            return

        plan_obj = objects.registry.Plan.get_by_id(self.context,
                                                   db_obj.plan_id)

        artifacts = plan_obj.raw_content.get('artifacts', [])
        for arti in artifacts:
            if repo_utils.verify_artifact(arti, collab_url):
                self._build_artifact(assem=db_obj, artifact=arti,
                                     commit_sha=commit_sha,
                                     status_url=status_url)

    def update(self, id, data):
        """Modify a resource."""
        updated = objects.registry.Assembly.update_and_save(self.context,
                                                            id, data)
        return updated

    def delete(self, id):
        """Delete a resource."""
        db_obj = objects.registry.Assembly.get_by_uuid(self.context, id)

        conductor_api.API(context=self.context).update_assembly(
            db_obj.id, {'status': ASSEMBLY_STATES.DELETING})

        deploy_api.API(context=self.context).destroy_assembly(
            assem_id=db_obj.id)

    def create(self, data, commit_sha='', status_url=None):
        """Create a new resource."""
        if 'workflow' in data and isinstance(data['workflow'], list):
            data['workflow'] = list(set(data['workflow']))
        else:
            data['workflow'] = ['unittest', 'build', 'deploy']
        db_obj = objects.registry.Assembly()
        db_obj.update(data)
        db_obj.uuid = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id
        db_obj.username = self.context.user_name

        db_obj.status = ASSEMBLY_STATES.QUEUED
        db_obj.create(self.context)

        plan_obj = objects.registry.Plan.get_by_id(self.context,
                                                   db_obj.plan_id)

        artifacts = plan_obj.raw_content.get('artifacts', [])
        for arti in artifacts:
            self._build_artifact(assem=db_obj, artifact=arti,
                                 commit_sha=commit_sha,
                                 status_url=status_url)
        return db_obj

    def _build_artifact(self, assem, artifact, verb='launch_workflow',
                        commit_sha='', status_url=None):

        # This is a tempory hack so we don't need the build client
        # in the requirements.
        image = objects.registry.Image()
        image.name = artifact['name']
        image.source_uri = artifact['content']['href']
        image.base_image_id = artifact.get('language_pack', 'auto')
        image.source_format = artifact.get('artifact_type',
                                           CONF.api.source_format)
        image.image_format = CONF.api.image_format
        image.uuid = uuidutils.generate_uuid()
        image.user_id = self.context.user
        image.project_id = self.context.project_id
        image.status = IMAGE_STATES.QUEUED
        image.create(self.context)
        test_cmd = artifact.get('unittest_cmd')
        run_cmd = artifact.get('run_cmd')
        repo_token = artifact.get('repo_token')
        ports = artifact.get('ports', [80])

        git_info = {
            'source_url': image.source_uri,
            'commit_sha': commit_sha,
            'repo_token': repo_token,
            'status_url': status_url
        }

        if test_cmd:
            repo_utils.send_status(0, status_url, repo_token, pending=True)

        worker_api.API(context=self.context).build_app(
            verb=verb,
            build_id=image.id,
            git_info=git_info,
            ports=ports,
            name=image.name,
            base_image_id=image.base_image_id,
            source_format=image.source_format,
            image_format=image.image_format,
            assembly_id=assem.id,
            workflow=assem.workflow,
            test_cmd=test_cmd,
            run_cmd=run_cmd)

    def get_all(self):
        """Return all assemblies, based on the query provided."""
        return objects.registry.AssemblyList.get_all(self.context)



# Copyright 2014 - Numergy
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import infrastructure
from solum.api.handlers import infrastructure_handler
from solum.common import exception
from solum import objects


class InfrastructureStackController(rest.RestController):
    """Manages operations on a single stack."""

    def __init__(self, stack_id):
        super(InfrastructureStackController, self).__init__()
        self._id = stack_id

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(infrastructure.InfrastructureStack)
    def get(self):
        """Return this stack."""
        handler = infrastructure_handler.InfrastructureStackHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return infrastructure.InfrastructureStack.from_db_model(
            handler.get(self._id), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(infrastructure.InfrastructureStack,
                         body=infrastructure.InfrastructureStack)
    def put(self, data):
        """Modify this stack."""
        handler = infrastructure_handler.InfrastructureStackHandler(
            pecan.request.security_context)
        res = handler.update(self._id,
                             data.as_dict(
                                 objects.registry.InfrastructureStack))
        host_url = pecan.request.application_url.rstrip('/')
        return infrastructure.InfrastructureStack.from_db_model(
            res, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=204)
    def delete(self):
        """Delete this stack."""
        handler = infrastructure_handler.InfrastructureStackHandler(
            pecan.request.security_context)
        return handler.delete(self._id)


class InfrastructureStacksController(rest.RestController):
    """Manages operations on the stacks collection."""

    @pecan.expose()
    def _lookup(self, stack_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return InfrastructureStackController(stack_id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(infrastructure.InfrastructureStack,
                         body=infrastructure.InfrastructureStack,
                         status_code=201)
    def post(self, data):
        """Create a new stack."""
        handler = infrastructure_handler.InfrastructureStackHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return infrastructure.InfrastructureStack.from_db_model(
            handler.create(data.as_dict(objects.registry.InfrastructureStack)),
            host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([infrastructure.InfrastructureStack])
    def get_all(self):
        """Return all stacks, based on the query provided."""
        handler = infrastructure_handler.InfrastructureStackHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [infrastructure.InfrastructureStack.from_db_model(
            assm, host_url) for assm in handler.get_all()]


class InfrastructureController(rest.RestController):
    """Infrastructure root controller."""

    stacks = InfrastructureStacksController()

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(infrastructure.Infrastructure)
    def index(self):
        base_url = pecan.request.application_url.rstrip('/')
        host_url = '%s/%s' % (base_url, 'v1')
        return infrastructure.Infrastructure(
            uri=host_url,
            name='solum',
            type='infrastructure',
            description='solum infrastructure endpoint',
            stacks_uri='%s/infrastructure/stacks' % host_url)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""rename image state column to status

Revision ID: 1f57763d7871
Revises: 2f15f41cd87f
Create Date: 2015-03-09 16:46:44.406091

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '1f57763d7871'
down_revision = '2f15f41cd87f'


def upgrade():
    op.alter_column('image', 'state',
                    new_column_name='status',
                    existing_nullable=True,
                    existing_type=sa.String(36))


def downgrade():
    op.alter_column('image', 'status',
                    new_column_name='state',
                    existing_nullable=True,
                    existing_type=sa.String(36))



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""API for interfacing with Solum Deployer."""

from oslo_config import cfg

from solum.common.rpc import service


class API(service.API):
    def __init__(self, context=None):
        cfg.CONF.import_opt('topic', 'solum.deployer.config',
                            group='deployer')
        super(API, self).__init__(context,
                                  topic=cfg.CONF.deployer.topic)

    def deploy(self, assembly_id, image_loc, image_name, ports):
        self._cast('deploy', assembly_id=assembly_id, image_loc=image_loc,
                   image_name=image_name, ports=ports)

    def scale(self, assembly_id):
        self._cast('scale', assembly_id=assembly_id)

    def destroy_assembly(self, assem_id):
        self._cast('destroy_assembly', assem_id=assem_id)

    def destroy_app(self, app_id):
        self._cast('destroy_app', app_id=app_id)



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import re

from oslo_config import cfg
from oslo_log import log as logging
import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import app
from solum.api.controllers.v1 import workflow
from solum.api.handlers import app_handler
from solum.common import exception
from solum.common import request
from solum.common import yamlutils
from solum import objects

LOG = logging.getLogger(__name__)


class AppController(rest.RestController):
    """Manages operations on a single app."""

    def __init__(self, app_id):
        super(AppController, self).__init__()
        self._id = app_id

    @exception.wrap_wsme_pecan_controller_exception
    @pecan.expose()
    def _lookup(self, resource, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        if resource == 'workflows':
            return workflow.WorkflowsController(self._id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(app.App)
    def get(self):
        """Return this app."""
        request.check_request_for_https()
        handler = app_handler.AppHandler(pecan.request.security_context)
        app_model = handler.get(self._id)
        host_url = pecan.request.application_url.rstrip('/')
        app_model = app.App.from_db_model(app_model, host_url)
        return app_model

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(app.App, body=app.App, status_code=200)
    def patch(self, data):
        """Modify this app."""
        request.check_request_for_https()
        handler = app_handler.AppHandler(pecan.request.security_context)
        handler.get(self._id)

        if not data:
            raise exception.BadRequest(reason="No body detected")

        updated_app = handler.patch(self._id, data)
        host_url = pecan.request.application_url.rstrip('/')
        updated_app = app.App.from_db_model(updated_app, host_url)

        return updated_app

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=202)
    def delete(self):
        """Delete this app."""
        handler = app_handler.AppHandler(pecan.request.security_context)
        handler.delete(self._id)


class AppsController(rest.RestController):
    """Manages operations on the apps collection."""

    def _validate(self, app_data):
        # check max apps created for given tenant
        handler = app_handler.AppHandler(pecan.request.security_context)
        if len(handler.get_all()) >= cfg.CONF.api.max_apps_per_tenant:
            msg = "Cannot create application as maximum allowed limit reached."
            raise exception.ResourceLimitExceeded(reason=msg)

        if not app_data.languagepack:
            raise exception.BadRequest(reason="Languagepack not specified.")

        if not app_data.name:
            raise exception.BadRequest(reason='App name cannot be empty.')

        msg = ("Application name must be 1-100 characters long, only contain "
               "a-z,0-9,-,_ and start with an alphabet character.")
        # check if app name contains any invalid characters
        if not app_data.name or not app_data.name[0].isalpha():
            raise exception.BadRequest(reason=msg)

        try:
            re.match(r'^([a-z0-9-_]{1,100})$', app_data.name).group(0)
        except AttributeError:
            raise exception.BadRequest(reason=msg)

        msg = "Application description must be less than 255 characters."
        if app_data.description and len(app_data.description) > 255:
            raise exception.BadRequest(reason=msg)

        # check if languagepack exists or not
        if str(app_data.languagepack).lower() != "false":
            try:
                objects.registry.Image.get_lp_by_name_or_uuid(
                    pecan.request.security_context,
                    app_data.languagepack,
                    include_operators_lp=True)
            except exception.ResourceNotFound:
                raise exception.ObjectNotFound(name="Languagepack",
                                               id=app_data.languagepack)

    @pecan.expose()
    def _lookup(self, app_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return AppController(app_id), remainder

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(app.App, body=app.App, status_code=201)
    def post(self, data):
        """Create a new app."""
        request.check_request_for_https()
        if not data:
            raise exception.BadRequest(reason='No data.')

        self._validate(data)

        handler = app_handler.AppHandler(pecan.request.security_context)

        app_data = data.as_dict(app.App)

        try:
            raw_content = yamlutils.load(pecan.request.body)
        except ValueError:
            try:
                raw_content = json.loads(pecan.request.body)
            except ValueError as exp:
                LOG.exception(exp)
                raise exception.BadRequest(reason='Invalid app data.')

        app_data['raw_content'] = json.dumps(raw_content)

        new_app = handler.create(app_data)
        host_url = pecan.request.application_url.rstrip('/')
        created_app = app.App.from_db_model(new_app, host_url)
        return created_app

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([app.App])
    def get_all(self):
        """Return all apps, based on the query provided."""
        request.check_request_for_https()
        handler = app_handler.AppHandler(pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        all_apps = [app.App.from_db_model(obj, host_url)
                    for obj in handler.get_all()]
        return all_apps



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Starter script for solum-db-manage."""

import os

from oslo_config import cfg
from oslo_db import options
from oslo_db.sqlalchemy.migration_cli import manager

CONF = cfg.CONF


def do_version(mgr):
    print('Current DB revision is %s' % mgr.version())


def do_upgrade(mgr):
    mgr.upgrade(CONF.command.revision)


def do_downgrade(mgr):
    mgr.downgrade(CONF.command.revision)


def do_stamp(mgr):
    mgr.stamp(CONF.command.revision)


def do_revision(mgr):
    mgr.revision(message=CONF.command.message,
                 autogenerate=CONF.command.autogenerate)


def add_command_parsers(subparsers):
    parser = subparsers.add_parser('version')
    parser.set_defaults(func=do_version)

    parser = subparsers.add_parser('upgrade')
    parser.add_argument('revision', nargs='?')
    parser.set_defaults(func=do_upgrade)

    parser = subparsers.add_parser('downgrade')
    parser.add_argument('revision', nargs='?')
    parser.set_defaults(func=do_downgrade)

    parser = subparsers.add_parser('stamp')
    parser.add_argument('revision', nargs='?')
    parser.set_defaults(func=do_stamp)

    parser = subparsers.add_parser('revision')
    parser.add_argument('-m', '--message')
    parser.add_argument('--autogenerate', action='store_true')
    parser.set_defaults(func=do_revision)


def get_manager():
    if cfg.CONF.database.connection is None:
        raise ValueError(
            'Database connection not set in /etc/solum/solum.conf')

    alembic_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__),
                     '..', 'objects', 'sqlalchemy',
                     'migration', 'alembic.ini'))
    migrate_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__),
                     '..', 'objects', 'sqlalchemy',
                     'migration', 'alembic_migrations'))
    migration_config = {'alembic_ini_path': alembic_path,
                        'alembic_repo_path': migrate_path,
                        'db_url': CONF.database.connection}
    return manager.MigrationManager(migration_config)


def main():
    command_opt = cfg.SubCommandOpt('command',
                                    title='Command',
                                    help='Available commands',
                                    handler=add_command_parsers)
    CONF.register_cli_opt(command_opt)

    # set_defaults() is called to register the db options.
    options.set_defaults(CONF)

    CONF(project='solum')
    CONF.command.func(get_manager())



# Copyright 2014 - Numergy
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.handlers import infrastructure_handler as infra
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestInfrastructureStackHandler(base.BaseTestCase):

    def setUp(self):
        super(TestInfrastructureStackHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_get(self, mock_registry):
        mock_registry.InfrastructureStack.get_by_uuid.return_value = {}
        handler = infra.InfrastructureStackHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        mock_registry.InfrastructureStack.get_by_uuid.assert_called_once_with(
            self.ctx, 'test_id')

    def test_get_all(self, mock_registry):
        mock_registry.StackList.get_all.return_value = {}
        handler = infra.InfrastructureStackHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.InfrastructureStackList.get_all.assert_called_once_with(
            self.ctx)

    def test_update(self, mock_reg):
        data = {'user_id': 'new_user_id',
                'image_id': 'new_image_id',
                'heat_stack_id': 'new_stack_id'}
        handler = infra.InfrastructureStackHandler(self.ctx)
        handler.update('test_id', data)
        mock_reg.InfrastructureStack.update_and_save.assert_called_once_with(
            self.ctx, 'test_id', data)

    @mock.patch('solum.common.clients.OpenStackClients')
    @mock.patch('solum.common.catalog.get')
    def test_create(self, mock_get, mock_clients, mock_registry):
        data = {'user_id': 'new_user_id',
                'image_id': 'new_image_id'}
        db_obj = fakes.FakeInfrastructureStack()
        fake_template = json.dumps({'description': 'test'})
        mock_get.return_value = fake_template
        parameters = {'image': 'new_image_id'}
        mock_registry.InfrastructureStack.return_value = db_obj
        mock_create = mock_clients.return_value.heat.return_value.stacks.create
        mock_create.return_value = {"stack": {"id": "fake_id",
                                    "links": [{"href": "http://fake.ref",
                                               "rel": "self"}]}}
        mock_queue = mock_clients.return_value.zaqar.return_value.queue
        handler = infra.InfrastructureStackHandler(self.ctx)
        res = handler.create(data)
        db_obj.update.assert_called_once_with(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)
        mock_create.assert_called_once_with(stack_name='infra',
                                            template=fake_template,
                                            parameters=parameters)
        mock_queue.assert_called_once_with(db_obj.uuid)

    @mock.patch('solum.common.clients.OpenStackClients')
    def test_create_zaqar_queue(self, mock_clients, mock_registry):
        queue_name = 'test'
        mock_queue = mock_clients.return_value.zaqar.return_value.queue
        handler = infra.InfrastructureStackHandler(self.ctx)
        handler._create_zaqar_queue(queue_name)
        mock_queue.assert_called_once_with(queue_name)

    def test_delete(self, mock_registry):
        db_obj = fakes.FakeInfrastructureStack()
        mock_registry.InfrastructureStack.get_by_uuid.return_value = db_obj
        handler = infra.InfrastructureStackHandler(self.ctx)
        handler.delete('test_id')
        db_obj.destroy.assert_called_once_with(self.ctx)
        mock_registry.InfrastructureStack.get_by_uuid.assert_called_once_with(
            self.ctx, 'test_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp.v1_1 import plans
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.camp.plan_handler.PlanHandler')
class TestPlans(base.BaseTestCase):
    def setUp(self):
        super(TestPlans, self).setUp()
        objects.load()

    def test_plans_get(self, PlanHandler, resp_mock, request_mock):
        hand_get_all = PlanHandler.return_value.get_all
        fake_plan = fakes.FakePlan()
        hand_get_all.return_value = [fake_plan]

        resp = plans.PlansController().get()
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp['result'].plan_links)
        plan_links = resp['result'].plan_links
        self.assertEqual(1, len(plan_links))
        self.assertEqual(fake_plan.name, plan_links[0].target_name)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Config options for Solum Conductor service."""


from oslo_config import cfg

SERVICE_OPTS = [
    cfg.StrOpt('topic',
               default='solum-conductor',
               help='The queue to add conductor tasks to'),
    cfg.HostAddressOpt('host',
                       default='localhost',
                       help='The location of the conductor rpc queue'),
]

opt_group = cfg.OptGroup(
    name='conductor',
    title='Options for the solum-conductor service')


def list_opts():
    yield opt_group, SERVICE_OPTS


cfg.CONF.register_group(opt_group)
cfg.CONF.register_opts(SERVICE_OPTS, opt_group)



# Copyright 2013 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.common import exception as solum_exception


class Handler(object):
    """The handler is responsible for fulfilling *one* request."""

    def __init__(self, context):
        super(Handler, self).__init__()
        self.context = context

    def get(self, id):
        """Return a resource."""
        excp = solum_exception.NotImplemented()
        raise excp

    def update(self, id, data):
        """Modify a resource."""
        excp = solum_exception.NotImplemented()
        raise excp

    def delete(self, id):
        """Delete a resource."""
        excp = solum_exception.NotImplemented()
        raise excp

    def create(self, data):
        """Create a new resource."""
        excp = solum_exception.NotImplemented()
        raise excp

    def get_all(self):
        """Return all resources, based on the query provided."""
        excp = solum_exception.NotImplemented()
        raise excp



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""create parameter table

Revision ID: 586c15b92842
Revises: 3a615b3b2eae
Create Date: 2014-10-28 17:53:56.752062

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

from oslo_utils import timeutils

# revision identifiers, used by Alembic.
revision = '586c15b92842'
down_revision = '3a615b3b2eae'


def upgrade():
    op.create_table(
        'parameter',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('plan_id', sa.Integer, sa.ForeignKey('plan.id'),
                  nullable=False),
        sa.Column('user_defined_params', models.YAMLEncodedDict(65535)),
        sa.Column('sys_defined_params', models.YAMLEncodedDict(65535)),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        )


def downgrade():
    op.drop_table('parameter')



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glanceclient import client as glanceclient
from heatclient import client as heatclient
from mistralclient.api import client as mistralclient
from neutronclient.neutron import client as neutronclient
from oslo_config import cfg
from swiftclient import client as swiftclient
from zaqarclient.queues.v1 import client as zaqarclient

from solum.common import exception
from solum.common import solum_barbicanclient
from solum.common import solum_keystoneclient
from solum.i18n import _


GLOBAL_CLIENT_OPTS = [
    cfg.StrOpt('region_name',
               default='RegionOne',
               help=_(
                   'Region of endpoint in Identity service catalog to use'
                   ' for all clients.')),
]

barbican_client_opts = [
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate for barbican "
                       "will not be verified.")), ]

# Note: this config is duplicated in many projects that use OpenStack
# clients. This should really be in the client.
# There is a place holder bug here:
# https://bugs.launchpad.net/solum/+bug/1292334
# that we use to track this.
glance_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Identity service catalog to use '
                   'for communication with the Glance service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.'))]

heat_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Identity service catalog to use '
                   'for communication with the OpenStack service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.')),
    cfg.StrOpt('ca_file',
               help=_('Optional CA cert file to use in SSL connections.')),
    cfg.StrOpt('cert_file',
               help=_('Optional PEM-formatted certificate chain file.')),
    cfg.StrOpt('key_file',
               help=_('Optional PEM-formatted file that contains the '
                      'private key.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate will not "
                       "be verified."))]

zaqar_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Queue service catalog to use '
                   'for communication with the Zaqar service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate for zaqar "
                       "will not be verified."))]

neutron_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Identity service catalog to use '
                   'for communication with the Neutron service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.')),
    cfg.StrOpt('ca_cert',
               help=_('Optional CA bundle file to use in SSL connections.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate for neutron "
                       "will not be verified."))]

swift_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Identity service catalog to use '
                   'for communication with the Swift service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.')),
    cfg.StrOpt('cacert',
               help=_('Optional CA cert file to use in SSL connections.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set the server certificate will not be verified."))]

mistral_client_opts = [
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_(
                   'Type of endpoint in Identity service catalog to use '
                   'for communication with the mistral service.')),
    cfg.StrOpt('region_name',
               default='',
               help=_(
                   'Region of endpoint in Identity service catalog to use.')),
    cfg.StrOpt('cacert',
               help=_('Optional CA cert file to use in SSL connections '
                      'with Mistral.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set the server certificate will not be verified "
                       "while using Mistral."))]


def list_opts():
    yield None, GLOBAL_CLIENT_OPTS
    yield 'barbican_client', barbican_client_opts
    yield 'glance_client', glance_client_opts
    yield 'heat_client', heat_client_opts
    yield 'zaqar_client', zaqar_client_opts
    yield 'neutron_client', neutron_client_opts
    yield 'swift_client', swift_client_opts
    yield 'mistral_client', mistral_client_opts


cfg.CONF.register_opts(GLOBAL_CLIENT_OPTS)
cfg.CONF.register_opts(barbican_client_opts, group='barbican_client')
cfg.CONF.register_opts(glance_client_opts, group='glance_client')
cfg.CONF.register_opts(heat_client_opts, group='heat_client')
cfg.CONF.register_opts(zaqar_client_opts, group='zaqar_client')
cfg.CONF.register_opts(neutron_client_opts, group='neutron_client')
cfg.CONF.register_opts(swift_client_opts, group='swift_client')
cfg.CONF.register_opts(mistral_client_opts, group='mistral_client')


def get_client_option(client, option):
    value = getattr(getattr(cfg.CONF, '%s_client' % client), option)
    if option == 'region_name':
        global_region = cfg.CONF.get(option)
        return value or global_region
    else:
        return value


class OpenStackClients(object):
    """Convenience class to create and cache client instances."""

    def __init__(self, context):
        self.context = context
        self._barbican = None
        self._keystone = None
        self._glance = None
        self._heat = None
        self._neutron = None
        self._zaqar = None
        self._mistral = None

    def url_for(self, **kwargs):
        return self.keystone().client.service_catalog.url_for(**kwargs)

    @property
    def auth_url(self):
        return self.keystone().endpoint

    @property
    def auth_token(self):
        return self.context.auth_token or self.keystone().auth_token

    @exception.wrap_keystone_exception
    def barbican(self):
        if self._barbican:
            return self._barbican

        insecure = get_client_option('barbican', 'insecure')
        self._barbican = solum_barbicanclient.BarbicanClient(
            verify=not insecure)
        return self._barbican

    def keystone(self):
        if self._keystone:
            return self._keystone

        self._keystone = solum_keystoneclient.KeystoneClient(self.context)
        return self._keystone

    @exception.wrap_keystone_exception
    def zaqar(self):
        if self._zaqar:
            return self._zaqar

        endpoint_type = get_client_option('zaqar', 'endpoint_type')
        region_name = get_client_option('zaqar', 'region_name')
        endpoint_url = self.url_for(service_type='queuing',
                                    interface=endpoint_type,
                                    region_name=region_name)
        conf = {'auth_opts':
                {'backend': 'keystone',
                 'options': {'os_auth_token': self.auth_token,
                             'os_auth_url': self.auth_url,
                             'insecure': get_client_option('zaqar',
                                                           'insecure')}
                 }
                }
        self._zaqar = zaqarclient.Client(endpoint_url, conf=conf)
        return self._zaqar

    @exception.wrap_keystone_exception
    def neutron(self):
        if self._neutron:
            return self._neutron

        endpoint_type = get_client_option('neutron', 'endpoint_type')
        region_name = get_client_option('neutron', 'region_name')
        endpoint_url = self.url_for(service_type='network',
                                    interface=endpoint_type,
                                    region_name=region_name)
        args = {
            'auth_url': self.auth_url,
            'endpoint_url': endpoint_url,
            'token': self.auth_token,
            'username': None,
            'password': None,
            'insecure': get_client_option('neutron', 'insecure'),
            'ca_cert': get_client_option('neutron', 'ca_cert')
        }
        self._neutron = neutronclient.Client('2.0', **args)
        return self._neutron

    @exception.wrap_keystone_exception
    def glance(self):
        if self._glance:
            return self._glance

        args = {
            'token': self.auth_token,
        }
        endpoint_type = get_client_option('glance', 'endpoint_type')
        region_name = get_client_option('glance', 'region_name')
        endpoint = self.url_for(service_type='image',
                                interface=endpoint_type,
                                region_name=region_name)
        self._glance = glanceclient.Client('2', endpoint, **args)

        return self._glance

    @exception.wrap_keystone_exception
    def mistral(self):
        if self._mistral:
            return self._mistral

        args = {
            'auth_token': self.auth_token,
        }
        endpoint_type = get_client_option('mistral', 'endpoint_type')
        region_name = get_client_option('mistral', 'region_name')
        endpoint = self.url_for(service_type='workflow',
                                interface=endpoint_type,
                                region_name=region_name)
        self._mistral = mistralclient.client(mistral_url=endpoint, **args)

        return self._mistral

    @exception.wrap_keystone_exception
    def heat(self, username=None, password=None, token=None):
        if self._heat:
            return self._heat

        if token:
            token_to_use = token
        else:
            token_to_use = self.auth_token

        endpoint_type = get_client_option('heat', 'endpoint_type')
        args = {
            'auth_url': self.auth_url,
            'token': token_to_use,
            'username': username,
            'password': password,
            'ca_file': get_client_option('heat', 'ca_file'),
            'cert_file': get_client_option('heat', 'cert_file'),
            'key_file': get_client_option('heat', 'key_file'),
            'insecure': get_client_option('heat', 'insecure')
        }

        region_name = get_client_option('heat', 'region_name')
        endpoint = self.url_for(service_type='orchestration',
                                interface=endpoint_type,
                                region_name=region_name)
        self._heat = heatclient.Client('1', endpoint, **args)

        return self._heat

    @exception.wrap_keystone_exception
    def swift(self):
        # Not caching swift connections because of range requests
        # Check how glance_store uses swift client for a reference
        endpoint_type = get_client_option('swift', 'endpoint_type')
        region_name = get_client_option('swift', 'region_name')
        args = {
            'auth_version': '2.0',
            'preauthtoken': self.auth_token,
            'preauthurl': self.url_for(service_type='object-store',
                                       interface=endpoint_type,
                                       region_name=region_name),
            'os_options': {'endpoint_type': endpoint_type,
                           'region_name': region_name},
            'cacert': get_client_option('swift', 'cacert'),
            'insecure': get_client_option('swift', 'insecure')
        }
        return swiftclient.Connection(**args)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1.datamodel import sensor as model
from solum.api.controllers.v1 import sensor as controller
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


class TestSensorValueTypeGood(base.BaseTestCase):

    scenarios = [
        ('int_str', dict(
            in_value=3, in_type='str', out_value='3')),
        ('int_int', dict(
            in_value=3, in_type='int', out_value=3)),
        ('str_int', dict(
            in_value='3', in_type='int', out_value=3)),
        ('float_str', dict(
            in_value=3.4, in_type='str', out_value='3.4')),
        ('str_float', dict(
            in_value='2.45', in_type='float', out_value=2.45)),
        ('float_float', dict(
            in_value=2.45, in_type='float', out_value=2.45)),
    ]

    def test_values(self):
        s = model.Sensor(sensor_type=self.in_type, value=self.in_value)
        self.assertEqual(self.out_value, s.value)


class TestSensorValueTypeBad(base.BaseTestCase):

    scenarios = [
        ('sp_int', dict(
            in_value=3.2, in_type='int')),
        ('bp_int', dict(
            in_value=3.7, in_type='int')),
        ('sn_int', dict(
            in_value=-3.1, in_type='int')),
        ('bn_int', dict(
            in_value=-3.9, in_type='int')),
        ('float', dict(
            in_value='sunny', in_type='float')),
    ]

    def test_values(self):
        s = model.Sensor(sensor_type=self.in_type, value=self.in_value)
        self.assertRaises(ValueError, getattr, s, 'value')


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.controllers.v1.sensor.sensor_handler.SensorHandler')
class TestSensorController(base.BaseTestCase):
    def setUp(self):
        super(TestSensorController, self).setUp()
        objects.load()

    def test_sensor_get(self, handler_mock, resp_mock,
                        request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        fake_sensor = fakes.FakeSensor()
        handler_get.return_value = fake_sensor
        obj = controller.SensorController('test_id')
        result = obj.get()
        self.assertIsNotNone(result)
        self.assertEqual(fake_sensor.name, result['result'].name)
        self.assertEqual(fake_sensor.documentation,
                         result['result'].documentation)
        self.assertEqual(fake_sensor.description, result['result'].description)
        self.assertEqual(fake_sensor.project_id, result['result'].project_id)
        self.assertEqual(fake_sensor.uuid, result['result'].uuid)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(result)
        handler_get.assert_called_once_with('test_id')

    def test_sensor_get_not_found(self, handler_mock, resp_mock,
                                  request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        handler_get.side_effect = exception.ResourceNotFound(
            name='sensor', sensor_id='test_id')
        obj = controller.SensorController('test_id')
        obj.get()
        self.assertEqual(404, resp_mock.status)
        handler_get.assert_called_once_with('test_id')

    def test_sensor_put(self, handler_mock, resp_mock,
                        request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'description': 'foo',
                       'value': '1234',
                       'name': 'test_name_changed'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_update = handler_mock.return_value.update
        handler_update.return_value = fakes.FakeSensor()
        obj = controller.SensorController('test_id')
        obj.put(fakes.FakeSensor())
        self.assertEqual(200, resp_mock.status)
        handler_update.assert_called_once_with('test_id', json_update)

    def test_sensor_put_none(self, handler_mock, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = None
        request_mock.content_type = 'application/json'
        handler_put = handler_mock.return_value.put
        handler_put.return_value = fakes.FakeSensor()
        controller.SensorController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_sensor_put_not_found(self, handler_mock, resp_mock,
                                  request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'hb42', 'value': '0'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_update = handler_mock.return_value.update
        handler_update.side_effect = exception.ResourceNotFound(
            name='sensor', sensor_id='test_id')
        controller.SensorController('test_id').put()
        handler_update.assert_called_with('test_id', json_update)
        self.assertEqual(404, resp_mock.status)

    def test_sensor_delete(self, mock_handler, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        handler_delete = mock_handler.return_value.delete
        handler_delete.return_value = None
        obj = controller.SensorController('test_id')
        obj.delete()
        handler_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)

    def test_sensor_delete_not_found(self, mock_handler,
                                     resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        handler_delete = mock_handler.return_value.delete
        handler_delete.side_effect = exception.ResourceNotFound(
            name='sensor', sensor_id='test_id')
        obj = controller.SensorController('test_id')
        obj.delete()
        handler_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.controllers.v1.sensor.sensor_handler.SensorHandler')
class TestSensorsController(base.BaseTestCase):
    def setUp(self):
        super(TestSensorsController, self).setUp()
        objects.load()

    def test_sensors_get_all(self, handler_mock, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get_all = handler_mock.return_value.get_all
        fake_sensor = fakes.FakeSensor()
        hand_get_all.return_value = [fake_sensor]
        obj = controller.SensorsController()
        resp = obj.get_all()
        self.assertIsNotNone(resp)
        self.assertEqual(fake_sensor.name, resp['result'][0].name)
        self.assertEqual(fake_sensor.documentation,
                         resp['result'][0].documentation)
        self.assertEqual(fake_sensor.description,
                         resp['result'][0].description)
        self.assertEqual(fake_sensor.project_id, resp['result'][0].project_id)
        self.assertEqual(fake_sensor.uuid, resp['result'][0].uuid)
        hand_get_all.assert_called_with()
        self.assertEqual(200, resp_mock.status)

    def test_sensors_post(self, handler_mock, resp_mock,
                          request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'value': '1234',
                       'name': 'test_name_changed',
                       'user_id': 'user_id',
                       'project_id': 'test_id',
                       'description': 'desc test',
                       'sensor_type': 'str',
                       'documentation': 'http://example.com/docs/blabla/',
                       'target_resource': 'http://example.com/target/'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeSensor()
        obj = controller.SensorsController()
        obj.post(fakes.FakeSensor())
        self.assertEqual(201, resp_mock.status)
        handler_create.assert_called_once_with(json_update)

    def test_sensors_post_nodata(self, handler_mock, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakeSensor()
        ret_val = controller.SensorsController().post()
        self.assertEqual("Missing argument: \"data\"",
                         str(ret_val['faultstring']))
        self.assertEqual(400, resp_mock.status)


class TestSensorAsDict(base.BaseTestCase):

    scenarios = [
        ('one', dict(data={'name': 'foo', 'value': '0'})),
        ('full', dict(data={'value': '1234',
                            'name': 'test_name_changed',
                            'user_id': 'user_id',
                            'project_id': 'test_id',
                            'description': 'desc test',
                            'sensor_type': 'str',
                            'documentation': 'http://example.com/docs/blabla/',
                            'target_resource': 'http://example.com/target/'}))
    ]

    def test_as_dict(self):
        objects.load()
        s = model.Sensor(**self.data)
        self.data.pop('uri', None)
        self.data.pop('type', None)
        self.assertEqual(self.data, s.as_dict(objects.registry.Sensor))



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Rename assembly_uuid column in userlogs

Revision ID: 176c26131889
Revises: 2c058b5051e4
Create Date: 2015-02-06 19:35:19.658096

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '176c26131889'
down_revision = '2c058b5051e4'


def upgrade():
    op.alter_column('userlogs', 'assembly_uuid',
                    new_column_name='resource_uuid',
                    existing_nullable=False,
                    existing_type=sa.String(36))
    op.add_column('userlogs',
                  sa.Column('resource_type', sa.String(length=36)))


def downgrade():
    op.alter_column('userlogs', 'resource_uuid',
                    new_column_name='assembly_uuid',
                    existing_nullable=False,
                    existing_type=sa.String(36))
    op.drop_column('userlogs', 'resource_type')



# Copyright 2014 - Numergy
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Infrastructure(api_types.Base):
    """Description of an Infrastructure."""

    stacks_uri = common_types.Uri
    "URI to services."

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/infrastructure',
                   name='infrastructure',
                   type='infrastructure',
                   stacks_uri='http://example.com/v1/infrastructure/stacks',
                   tags=['small'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='Solum Infrastructure endpoint')


class InfrastructureStack(api_types.Base):
    """Representation of an InfrastructureStack.

    An InfrastructureStack is a set of servers used by Solum for
    infrastructure purpose. It can be a build farm, a set of git servers, bug
    trackers, or more.
    """

    image_id = wtypes.text
    "Unique Identifier of the image"

    heat_stack_id = wtypes.text
    "Unique Identifier of the heat stack associated with this infra stack"

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/infrastructure/stacks/1234',
                   name='jenkins',
                   type='infrastructure_stack',
                   image_id='1234',
                   heat_stack_id='1234',
                   tags=['small'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='A jenkins build farm of servers')



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import jsonpatch
from oslo_db import exception as db_exc
from oslo_utils import uuidutils
import pecan
from pecan import rest
import wsme
from wsme.rest import json as wjson
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import plans as model
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.handlers.camp import plan_handler as plan_handler
from solum.common import exception
from solum.common import yamlutils
from solum.i18n import _


MAL_PATCH_ERR = 'JSON Patch request missing one or more required components'
UNSUP_VER_ERR = 'camp_version \'%s\' is not supported by this implementation'


def clean_plan(plan_dict):
    del plan_dict['camp_version']
    return plan_dict


def fluff_plan(plan_dict, pid):
    """Fluff the plan with a camp_version and uri."""
    plan_dict['camp_version'] = "CAMP 1.1"
    host_url = pecan.request.application_url.rstrip('/')
    plan_dict['uri'] = uris.PLAN_URI_STR % (host_url, pid)
    return plan_dict


class JsonPatchProcessingException(exception.SolumException):
    msg_fmt = _("Error while processing the JSON Patch document: %(reason)s")
    code = 500


class PlansController(rest.RestController):
    """CAMP v1.1 plans controller."""

    _custom_actions = {
        'patch': ['PATCH']
    }

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
    def delete(self, uuid):
        """Delete this plan."""
        handler = (plan_handler.
                   PlanHandler(pecan.request.security_context))
        try:
            handler.delete(uuid)
        except (db_exc.DBReferenceError, db_exc.DBError):
            raise exception.PlanStillReferenced(name=uuid)

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Plan, wtypes.text)
    def get_one(self, uuid):
        """Return the appropriate CAMP-style plan resource."""
        handler = (plan_handler.
                   PlanHandler(pecan.request.security_context))
        db_obj = handler.get(uuid)
        plan_dict = fluff_plan(db_obj.refined_content(), db_obj.uuid)
        return model.Plan(**plan_dict)

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Plans)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        puri = uris.PLANS_URI_STR % host_url
        pdef_uri = uris.DEPLOY_PARAMS_URI % host_url
        desc = "Solum CAMP API plans collection resource."

        handler = plan_handler.PlanHandler(pecan.request.security_context)
        plan_objs = handler.get_all()
        p_links = []
        for m in plan_objs:
            p_links.append(common_types.Link(href=uris.PLAN_URI_STR %
                                             (host_url, m.uuid),
                                             target_name=m.name))

        # if there aren't any plans, avoid returning a resource with an
        # empty plan_links array
        if len(p_links) > 0:
            res = model.Plans(uri=puri,
                              name='Solum_CAMP_plans',
                              type='plans',
                              description=desc,
                              parameter_definitions_uri=pdef_uri,
                              plan_links=p_links)
        else:
            res = model.Plans(uri=puri,
                              name='Solum_CAMP_plans',
                              type='plans',
                              description=desc,
                              parameter_definitions_uri=pdef_uri)
        return res

    @exception.wrap_pecan_controller_exception
    @pecan.expose('json', content_type='application/json-patch+json')
    def patch(self, uuid):
        """Patch an existing CAMP-style plan."""

        handler = (plan_handler.
                   PlanHandler(pecan.request.security_context))
        plan_obj = handler.get(uuid)

        # TODO(gilbert.pilz@oracle.com) check if there are any assemblies that
        # refer to this plan and raise an PlanStillReferenced exception if
        # there are.

        if not pecan.request.body or len(pecan.request.body) < 1:
            raise exception.BadRequest(reason='empty request body')

        # check to make sure the request has the right Content-Type
        if (pecan.request.content_type is None or
                pecan.request.content_type != 'application/json-patch+json'):
            raise exception.UnsupportedMediaType(
                name=pecan.request.content_type,
                method='PATCH')

        try:
            patch = jsonpatch.JsonPatch.from_string(pecan.request.body)
            patched_obj = patch.apply(plan_obj.refined_content())
            db_obj = handler.update(uuid, patched_obj)
        except KeyError:
            # a key error indicates one of the patch operations is missing a
            # component
            raise exception.BadRequest(reason=MAL_PATCH_ERR)
        except jsonpatch.JsonPatchConflict:
            raise exception.Unprocessable
        except jsonpatch.JsonPatchException as jpe:
            raise JsonPatchProcessingException(reason=str(jpe))

        return fluff_plan(db_obj.refined_content(), db_obj.uuid)

    @exception.wrap_pecan_controller_exception
    @pecan.expose('json', content_type='application/x-yaml')
    def post(self):
        """Create a new CAMP-style plan."""
        if not pecan.request.body or len(pecan.request.body) < 1:
            raise exception.BadRequest

        # check to make sure the request has the right Content-Type
        if (pecan.request.content_type is None or
                pecan.request.content_type != 'application/x-yaml'):
            raise exception.UnsupportedMediaType(
                name=pecan.request.content_type,
                method='POST')

        try:
            yaml_input_plan = yamlutils.load(pecan.request.body)
        except ValueError as excp:
            raise exception.BadRequest(reason='Plan is invalid. '
                                       + str(excp))

        camp_version = yaml_input_plan.get('camp_version')
        if camp_version is None:
            raise exception.BadRequest(
                reason='camp_version attribute is missing from submitted Plan')
        elif camp_version != 'CAMP 1.1':
            raise exception.BadRequest(reason=UNSUP_VER_ERR % camp_version)

        # Use Solum's handler as the point of commonality. We can do this
        # because Solum stores plans in the DB in their JSON form.
        handler = (plan_handler.
                   PlanHandler(pecan.request.security_context))
        model_plan = model.Plan(**yaml_input_plan)

        # Move any inline Service Specifications to the "services" section.
        # This avoids an issue where WSME can't properly handle multi-typed
        # attributes (e.g. 'fulfillment'). It also smoothes out the primary
        # difference between CAMP plans and Solum plans, namely that Solum
        # plans don't have inline Service Specifications.
        for art in model_plan.artifacts:
            if art.requirements != wsme.Unset:
                for req in art.requirements:
                    if (req.fulfillment != wsme.Unset and
                        isinstance(req.fulfillment,
                                   model.ServiceSpecification)):
                        s_spec = req.fulfillment

                        # if the inline service spec doesn't have an id
                        # generate one
                        if s_spec.id == wsme.Unset:
                            s_spec.id = uuidutils.generate_uuid()

                        # move the inline service spec to the 'services'
                        # section
                        if model_plan.services == wsme.Unset:
                            model_plan.services = [s_spec]
                        else:
                            model_plan.services.append(s_spec)

                        # set the fulfillment to the service spec id
                        req.fulfillment = "id:%s" % s_spec.id

        db_obj = handler.create(clean_plan(wjson.tojson(model.Plan,
                                                        model_plan)))
        plan_dict = fluff_plan(db_obj.refined_content(), db_obj.uuid)

        pecan.response.status = 201
        pecan.response.location = plan_dict['uri']
        return plan_dict



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import extensions as model
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.handlers import extension_handler
from solum.common import exception


uri_string = '%s/camp/v1_1/extensions/'
description_string = "Solum CAMP API extensions collection resource."


class ExtensionsController(rest.RestController):
    """CAMP v1.1 extensions controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Extensions)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        euri = uris.EXTNS_URI_STR % host_url
        desc = "Solum CAMP API extensions collection resource."

        handlr = (extension_handler.
                  ExtensionHandler(pecan.request.security_context))
        ext_objs = handlr.get_all()
        e_links = []
        for m in ext_objs:
            e_links.append(common_types.Link(href=uris.EXTN_URI_STR %
                                             (host_url, m.uuid),
                                             target_name=m.name))

        # if there aren't any extensions, avoid returning a resource with an
        # empty extension_links array
        if len(e_links) > 0:
            res = model.Extensions(uri=euri,
                                   name="Solum_CAMP_extensions",
                                   type='extensions',
                                   description=desc,
                                   extension_links=e_links)
        else:
            res = model.Extensions(uri=euri,
                                   name="Solum_CAMP_extensions",
                                   type='extensions',
                                   description=desc)
        return res



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Common functionality for the application object model

The object model must be initialized at service start via

  solum.objects.load()

and all objects should be retrieved via

  solum.objects.registry.<class>

in application code.
"""

from oslo_config import cfg
from oslo_db import api

from solum.objects import registry as registry_mod

db_opts = [
    cfg.StrOpt('schema_mode',
               default='new',
               help="The version of the schema that should be "
                    "running: 'old', 'transition', 'new'")
]


def list_opts():
    yield "database", db_opts


CONF = cfg.CONF
CONF.register_opts(db_opts, "database")

_BACKEND_MAPPING = {'sqlalchemy': 'solum.objects.sqlalchemy'}
IMPL = api.DBAPI.from_config(CONF, backend_mapping=_BACKEND_MAPPING)


def transition_schema():
    """Is the new schema in write-only mode."""
    return cfg.CONF.database.schema_mode == 'transition'


def new_schema():
    """Should objects be writing to the new schema."""
    return cfg.CONF.database.schema_mode != 'old'


def load():
    """Ensure that the object model is initialized."""
    global registry
    registry.clear()
    IMPL.load()


registry = registry_mod.Registry()



# Copyright 2013 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import operation
from solum.api.controllers.v1.datamodel import sensor
from solum.api.controllers.v1.datamodel import service
from solum.api.controllers.v1.datamodel import types as api_types


class Component(api_types.Base):
    """The Component resource represents one part of an Assembly.

    For example, an instance of a database service may be a
    Component. A Component resource may also represent a static artifact, such
    as an archive file that contains data for initializing your application.
    An Assembly may have different components that represent different
    processes that run. For example, you may have one Component that represents
    an API service process, and another that represents a web UI process that
    consumes that API service. The simplest case is when an Assembly has only
    one component. For example, your component may be named "PHP" and refers to
    the PHP Service offered by the platform for running a PHP application.
    """

    assembly_uuid = wtypes.text
    """"The uuid of the assembly that this component belongs in."""

    services = [service.Service]
    """Services that belong to the component."""

    operations = [operation.Operation]
    """Operations that belong to the component."""

    sensors = [sensor.Sensor]
    """Sensors that belong to the component."""

    abbreviated = bool
    """Boolean value indicating if this components has nested components at
    more than one level of depth."""

    components_ids = [wtypes.text]
    """IDs of nested component of the component."""

    resource_uri = common_types.Uri
    """Remote resource URI of the component."""

    plan_uri = common_types.Uri
    """URI of Plan of which the component is a part."""

    component_type = wtypes.text
    """Type of component e.g. heat_stack."""

    heat_stack_id = wtypes.text
    """Unique identifier of the Heat Stack."""

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/components/php-web-app',
                   name='php-web-app',
                   type='component',
                   component_type='heat_stack',
                   description='A php web application component',
                   tags=['group_xyz'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   assembly_id='b3e0d79c698ea7b1561075bcfbbd2206a23d19b9',
                   heat_stack_id='4c712026-dcd5-4664-90b8-0915494c1332',
                   abbreviated=True,
                   components_ids=[],
                   services=[],
                   operations=[],
                   sensors=[])



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import cfg

from solum.api.handlers import language_pack_handler
from solum.common import exception as exc
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry.Image')
class TestLanguagePackHandler(base.BaseTestCase):
    def setUp(self):
        super(TestLanguagePackHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_languagepack_get(self, mock_img):
        mock_img.get_lp_by_name_or_uuid.return_value = {}
        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        mock_img.get_lp_by_name_or_uuid.assert_called_once_with(
            self.ctx, 'test_id', include_operators_lp=True)

    def test_languagepack_get_all(self, mock_img):
        mock_img.get_by_uuid.return_value = {}
        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_img.get_all_languagepacks.assert_called_once_with(
            self.ctx)

    @mock.patch('solum.api.handlers.language_pack_handler.'
                'LanguagePackHandler._start_build')
    def test_languagepack_create(self, mock_lp_build, mock_img):
        data = {'name': 'new app',
                'source_uri': 'git@github.com/foo/foo.git'}
        fi = fakes.FakeImage()
        cfg.CONF.set_override('max_languagepack_limit', '10',
                              group='api')
        mock_img.get_num_of_lps.return_value = 1

        mock_img.get_lp_by_name_or_uuid.side_effect = exc.ResourceNotFound()
        mock_img.return_value = fi
        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        res = handler.create(data, lp_metadata=None, lp_params=None)
        mock_lp_build.assert_called_once_with(res, None)
        fi.update.assert_called_once_with(data)
        fi.create.assert_called_once_with(self.ctx)

    @mock.patch('solum.api.handlers.language_pack_handler.'
                'LanguagePackHandler._start_build')
    def test_languagepack_create_limit_reached(self, mock_lp_build,
                                               mock_img):

        data = {'name': 'new app1',
                'source_uri': 'git@github.com/foo/foo.git'}
        cfg.CONF.set_override('max_languagepack_limit', '1',
                              group='api')
        mock_img.get_num_of_lps.return_value = 1

        handler = language_pack_handler.LanguagePackHandler(self.ctx)

        self.assertRaises(exc.ResourceLimitExceeded, handler.create,
                          data, lp_metadata=None, lp_params=None)

    def test_lp_create_bad_git_url(self, mock_img):

        invalid_urls_list = list()
        invalid_urls_list.append('http://github.com/skdhfskjhdks')
        invalid_urls_list.append('github.com/abc/xyz')
        invalid_urls_list.append('xyz://github.com/abc/xyz.git')
        invalid_urls_list.append('xyz://github.com/abc/xyz')
        invalid_urls_list.append('abc')
        invalid_urls_list.append('http')
        invalid_urls_list.append('git')

        cfg.CONF.set_override('max_languagepack_limit', '10',
                              group='api')
        mock_img.get_num_of_lps.return_value = 1

        for invalid_url in invalid_urls_list:
            data = {'name': 'new app2',
                    'source_uri': invalid_url}
            handler = language_pack_handler.LanguagePackHandler(self.ctx)

            self.assertRaises(exc.BadRequest, handler.create, data,
                              lp_metadata=None, lp_params=None)

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.objects.registry.PlanList')
    @mock.patch('solum.objects.sqlalchemy.app.App')
    def test_languagepack_delete(self, mock_app, mock_planlist,
                                 mock_log_handler,
                                 mock_swift_delete, mock_img):

        cfg.CONF.worker.image_storage = "swift"
        fi = fakes.FakeImage()
        mock_img.get_lp_by_name_or_uuid.return_value = fi
        mock_img.destroy.return_value = {}
        mock_planlist.get_all.return_value = {}

        mock_app.get_all_by_lp.return_value = {}

        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        handler.delete('test_lp')

        mock_img.get_lp_by_name_or_uuid.assert_called_once_with(
            self.ctx, 'test_lp')
        docker_image_name = fi.docker_image_name
        img_filename = docker_image_name.split('-', 1)[1]
        mock_swift_delete.assert_called_once_with('solum_lp', img_filename)
        log_handler = mock_log_handler.return_value
        log_handler.delete.assert_called_once_with(fi.uuid)
        fi.destroy.assert_called_once_with(self.ctx)

    @mock.patch('solum.objects.registry.PlanList')
    def test_languagepack_delete_with_plan_using_lp(self, mock_planlist,
                                                    mock_img):
        fi = fakes.FakeImage()
        fi.name = 'test_lp'
        mock_img.get_lp_by_name_or_uuid.return_value = fi
        mock_img.destroy.return_value = {}
        mock_planlist.get_all.return_value = [fakes.FakePlan()]
        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        self.assertRaises(exc.LPStillReferenced, handler.delete, 'test_lp')
        mock_img.get_lp_by_name_or_uuid.assert_called_once_with(
            self.ctx, 'test_lp')
        self.assertTrue(mock_planlist.get_all.called)
        assert not fi.destroy.called

    @mock.patch('solum.common.solum_swiftclient.SwiftClient.delete_object')
    @mock.patch('solum.api.handlers.userlog_handler.UserlogHandler')
    @mock.patch('solum.objects.registry.PlanList')
    @mock.patch('solum.objects.sqlalchemy.app.App')
    def test_languagepack_delete_with_plan_not_using_lp(self,
                                                        mock_app,
                                                        mock_planlist,
                                                        mock_log_handler,
                                                        mock_swift_delete,
                                                        mock_img):
        cfg.CONF.worker.image_storage = "swift"
        fi = fakes.FakeImage()
        mock_img.get_lp_by_name_or_uuid.return_value = fi
        mock_img.destroy.return_value = {}
        mock_planlist.get_all.return_value = [fakes.FakePlan()]

        mock_app.get_all_by_lp.return_value = {}
        handler = language_pack_handler.LanguagePackHandler(self.ctx)
        handler.delete('lp_name')

        mock_img.get_lp_by_name_or_uuid.assert_called_once_with(
            self.ctx, 'lp_name')
        docker_image_name = fi.docker_image_name
        img_filename = docker_image_name.split('-', 1)[1]
        mock_swift_delete.assert_called_once_with('solum_lp', img_filename)
        self.assertTrue(mock_planlist.get_all.called)
        log_handler = mock_log_handler.return_value
        log_handler.delete.assert_called_once_with(fi.uuid)
        fi.destroy.assert_called_once_with(self.ctx)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import userlog
from solum.tests import base
from solum.tests import utils

from oslo_utils import uuidutils


class TestUserlog(base.BaseTestCase):
    def setUp(self):
        super(TestUserlog, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()
        a_id = uuidutils.generate_uuid()

        self.data = [{'id': 12,
                      'resource_uuid': '%s' % a_id,
                      'resource_type': 'app',
                      'strategy': 'local',
                      'location': '/dev/null',
                      'project_id': self.ctx.project_id,
                      'strategy_info': '{}',
                      }]
        utils.create_models_from_data(userlog.Userlog, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Userlog)
        self.assertTrue(registry.UserlogList)

    def test_get_all(self):
        lst = userlog.UserlogList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        ulog = userlog.Userlog().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(ulog, key))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import yaml

from solum.common import yamlutils
from solum.tests import base


class TestYamlUtils(base.BaseTestCase):
    def setUp(self):
        super(TestYamlUtils, self).setUp()

    def test_load_yaml(self):
        yml_dict = yamlutils.load('a: x\nb: y\n')
        self.assertEqual({'a': 'x', 'b': 'y'}, yml_dict)

    def test_load_empty_yaml(self):
        self.assertRaises(ValueError, yamlutils.load, '{}')

    def test_load_empty_list(self):
        yml_dict = yamlutils.load('[]')
        self.assertEqual([], yml_dict)

    def test_load_invalid_yaml_syntax(self):
        self.assertRaises(ValueError, yamlutils.load, "}invalid: y'm'l3!")

    def test_load_invalid_yaml_type(self):
        self.assertRaises(ValueError, yamlutils.load, 'invalid yaml type')

    @mock.patch('solum.common.yamlutils.yaml.dump')
    def test_dump_yaml(self, dump):
        if hasattr(yaml, 'CSafeDumper'):
            yaml_dumper = yaml.CSafeDumper
        else:
            yaml_dumper = yaml.SafeDumper
        yamlutils.dump('version: 1')
        dump.assert_called_with('version: 1', Dumper=yaml_dumper)



# Copyright 2015 - Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy as sa
from sqlalchemy.orm import exc as orm_exc
from sqlalchemy.sql import text

from solum.objects.sqlalchemy import assembly
from solum.objects.sqlalchemy import component
from solum.objects.sqlalchemy import image
from solum.objects.sqlalchemy import models as sql
from solum.objects.sqlalchemy import plan
from solum.objects import workflow as abstract


class Workflow(sql.Base, abstract.Workflow):
    """Represent an app workflow in sqlalchemy."""

    __tablename__ = 'workflow'
    __resource__ = 'workflows'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.String(36), primary_key=True)
    deleted = sa.Column(sa.Boolean, default=False)
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))

    app_id = sa.Column(sa.String(36))
    wf_id = sa.Column(sa.Integer())
    source = sa.Column(sql.YAMLEncodedDict(1024))
    config = sa.Column(sql.YAMLEncodedDict(1024))
    actions = sa.Column(sql.YAMLEncodedDict(1024))
    assembly = sa.Column(sa.Integer())
    status = sa.Column(sa.String(36))
    result = sa.Column(sa.Text)

    def _non_updatable_fields(self):
        return set(('id', 'project_id'))

    @classmethod
    def get_by_assembly_id(cls, assembly_id):
        try:
            session = sql.Base.get_session()
            result = session.query(Workflow)
            result = result.filter(Workflow.assembly == assembly_id)
            return result.one()
        except orm_exc.NoResultFound:
            cls._raise_not_found(assembly_id)

    @classmethod
    def get_by_uuid(cls, context, item_uuid):
        return cls.get_by_id(context, item_uuid)

    @classmethod
    @sql.retry
    def update_and_save(cls, context, id_or_uuid, data):
        try:
            session = sql.SolumBase.get_session()
            with session.begin():
                query = session.query(cls).filter_by(id=id_or_uuid)
                obj = sql.filter_by_project(context, query).one()
                if obj._is_updatable():
                    obj.update(data)
                    session.merge(obj)
            return obj
        except orm_exc.NoResultFound:
            cls._raise_not_found(id_or_uuid)

    @classmethod
    @sql.retry
    def insert(cls, context, db_obj):
        try:
            ss = sql.Base.get_session()
            with ss.begin():
                s = text("select max(wf_id) as max_wf_id "
                         "from workflow where app_id like :app")
                res = ss.execute(s, params=dict(app=db_obj.app_id)).fetchone()
                if res.max_wf_id:
                    db_obj.wf_id = res.max_wf_id + 1
                else:
                    db_obj.wf_id = 1
                db_obj.create(context)
        except orm_exc.NoResultFound:
            cls._raise_not_found(db_obj.app_id)

    @classmethod
    @sql.retry
    def destroy(cls, app_id):
        # Delete relevant plan, assembly, component on workflow delete
        try:
            session = sql.Base.get_session()
            plan_id = None
            image_name = None
            with session.begin():
                wfs = session.query(cls).filter_by(app_id=app_id).all()
                for wf_obj in wfs:
                    asm = session.query(assembly.Assembly).filter_by(
                        id=wf_obj.assembly).one()
                    plan_id = asm.plan_id
                    image_name = asm.name
                    # delete component
                    session.query(component.Component).filter_by(
                        assembly_id=wf_obj.assembly).delete()
                    # delete assembly
                    session.query(assembly.Assembly).filter_by(
                        id=wf_obj.assembly).delete()

                # delete plan
                if not plan_id:
                    plan_id = app_id
                session.query(plan.Plan).filter_by(id=plan_id).delete()
                # delete workflows
                session.query(cls).filter_by(app_id=app_id).delete()
                # delete image
                if image_name:
                    session.query(image.Image).filter_by(
                        name=image_name).delete()
        except orm_exc.NoResultFound:
            cls._raise_not_found(app_id)


class WorkflowList(abstract.WorkflowList):
    """Represent a list of app workflows in sqlalchemy."""

    @classmethod
    def get_all(cls, context, app_id=None):
        wfs = sql.model_query(context, Workflow)
        if app_id is not None:
            wfs = wfs.filter_by(app_id=app_id)
        return WorkflowList(wfs)



# Copyright 2015 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import datetime
import functools
import re
import shutil
import subprocess
import time
import types


def retry(fun):
    """Decorator to retry a call."""
    @functools.wraps(fun)
    def _wrapper(*args, **kwargs):
        max_retries = kwargs.pop('max_retries', 2)
        expected = kwargs.pop('expected_ret_code', 0)

        actual_ret = None
        for tries in range(max_retries):
            try:
                actual_ret = fun(*args, **kwargs)
                if expected == actual_ret:
                    return actual_ret
            except Exception:
                if tries + 1 >= max_retries:
                    raise

            time.sleep(1)
        return actual_ret
    return _wrapper


def is_git_sha(revision):
    return re.match(r'^([a-f0-9]{40})$', revision)


def timestamp():
    return datetime.datetime.utcnow().strftime('%Y%m%dt%H%M%S%f')


def rm_tree(directory):
    shutil.rmtree(directory)


class Shell(object):
    """Helper methods to run shell commands."""

    @classmethod
    def run(cls, args, timeout=None):
        if timeout:
            prefix = ['timeout', '--signal=SIGKILL', str(timeout)]
            if isinstance(args, types.StringTypes):
                args = prefix.append(args)
            elif isinstance(args, list):
                args = prefix.extend(args)

        return subprocess.call(args)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from wsme import types as wtypes

from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Format(api_types.Base):
    """CAMP v1.1 format resource model."""

    mime_type = wtypes.text
    """This attribute contains the mime-type to be used by the Platform in
     HTTP [RFC2616] compliant content negotiation for this format."""

    version = wtypes.text
    """This attribute contains the version identifier of the data serialization
     format used."""

    documentation = common_types.Uri
    """ The value of this attribute is a URL reference to a document that
     describes this format."""

    def __init__(self, **kwds):
        super(Format, self).__init__(**kwds)

    def fix_uris(self, host_url):
        """Update URIs to reflect a host URL."""

        ret_val = copy.deepcopy(self)
        ret_val.uri = uris.FORMAT_URI_STR % (host_url, ret_val.uri)

        return ret_val



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""create workflow table

Revision ID: 390c6c4aa97b
Revises: 43df309dbf3f
Create Date: 2015-07-06 16:11:55.370993

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

from oslo_utils import timeutils

# revision identifiers, used by Alembic.
revision = '390c6c4aa97b'
down_revision = '43df309dbf3f'


def upgrade():
    op.create_table(
        'workflow',
        sa.Column('id', sa.String(length=36), primary_key=True,
                  nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('deleted_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('deleted', sa.Boolean, default=False, nullable=False),
        sa.Column('project_id', sa.String(length=36)),
        sa.Column('user_id', sa.String(length=36)),

        sa.Column('app_id', sa.String(length=36), nullable=False),
        sa.Column('wf_id', sa.Integer(), nullable=False),
        sa.Column('source', models.YAMLEncodedDict(1024)),
        sa.Column('config', models.YAMLEncodedDict(1024)),
        sa.Column('actions', models.YAMLEncodedDict(1024)),
        sa.Column('status', sa.String(length=36)),
        sa.Column('assembly', sa.Integer)
        )


def downgrade():
    op.drop_table('workflow')



# Copyright 2014 - Numergy
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils import uuidutils

from solum.api.handlers import handler
from solum.common import catalog
from solum.common import clients
from solum import objects


class InfrastructureStackHandler(handler.Handler):
    """Fulfills a request on the infrastructure stack resource."""

    def get(self, id):
        """Return a stack."""
        return objects.registry.InfrastructureStack.get_by_uuid(
            self.context, id)

    def update(self, id, data):
        """Modify a stack."""
        updated = objects.registry.InfrastructureStack.update_and_save(
            self.context, id, data)
        return updated

    def delete(self, id):
        """Delete a stack."""
        db_obj = objects.registry.InfrastructureStack.get_by_uuid(
            self.context, id)
        db_obj.destroy(self.context)

    def create(self, data):
        """Create a new stack.

        Create a new infrastructure stack by using Heat. Note that a zaqar
        queue is created and will be consumed by solum-infra-guestagent.
        """
        db_obj = objects.registry.InfrastructureStack()
        db_obj.update(data)
        db_obj.uuid = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id

        self._create_zaqar_queue(db_obj.uuid)
        db_obj.heat_stack_id = self._deploy_infra(data.get('image_id'))

        db_obj.create(self.context)
        return db_obj

    def _create_zaqar_queue(self, queue_name):
        osc = clients.OpenStackClients(self.context)
        osc.zaqar().queue(queue_name)

    def _deploy_infra(self, image_id):
        osc = clients.OpenStackClients(self.context)

        parameters = {'image': image_id}

        template = catalog.get('templates', 'infra')

        created_stack = osc.heat().stacks.create(stack_name='infra',
                                                 template=template,
                                                 parameters=parameters)
        return created_stack['stack']['id']

    def get_all(self):
        """Return all stacks, based on the query provided."""
        return objects.registry.InfrastructureStackList.get_all(self.context)



#    Copyright 2015 Intel, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import pycodestyle
from unittest import mock

import textwrap

from solum.hacking import checks
from solum.tests import base


class HackingTestCase(base.BaseTestCase):

    # We are patching pycodestyle so that only the check under test is actually
    # installed.

    @mock.patch('pycodestyle._checks',
                {'physical_line': {}, 'logical_line': {}, 'tree': {}})
    def _run_check(self, code, checker, filename=None):
        pycodestyle.register_check(checker)

        lines = textwrap.dedent(code).strip().splitlines(True)

        checker = pycodestyle.Checker(filename=filename, lines=lines)
        checker.check_all()
        checker.report._deferred_print.sort()
        return checker.report._deferred_print

    def _assert_has_errors(self, code, checker, expected_errors=None,
                           filename=None):
        actual_errors = [e[:3] for e in
                         self._run_check(code, checker, filename)]
        self.assertEqual(expected_errors or [], actual_errors)

    def _assert_has_no_errors(self, code, checker, filename=None):
        self._assert_has_errors(code, checker, filename=filename)

    def test_no_mutable_default_args(self):
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def get_info_from_bdm(virt_type, bdm, mapping=[])"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined = []"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined, undefined = [], {}"))))



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import cfg

import keystoneclient.exceptions as kc_exception  # noqa

from solum.common import exception
from solum.common import solum_keystoneclient
from solum.tests import base
from solum.tests import utils

cfg.CONF.import_group('keystone_authtoken',
                      'keystonemiddleware.auth_token')


@mock.patch('keystoneclient.v3.client.Client')
class KeystoneClientTest(base.BaseTestCase):
    """Test cases for solum.common.solum_keystoneclient."""

    def setUp(self):
        super(KeystoneClientTest, self).setUp()
        dummy_url = 'http://server.test:5000/v2.0'

        self.ctx = utils.dummy_context()
        self.ctx.auth_url = dummy_url
        self.ctx.auth_token = 'abcd1234'
        self.ctx.auth_token_info = None

        cfg.CONF.set_override('www_authenticate_uri', dummy_url,
                              group='keystone_authtoken')

    def test_init_v3_token(self, mock_ks):
        """Test creating the client, token auth."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.project_id = None
        self.ctx.trust_id = None
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        solum_ks_client.client
        self.assertIsNotNone(solum_ks_client._client)
        mock_ks.assert_called_once_with(token='abcd1234', project_id=None,
                                        auth_url='http://server.test:5000/v3',
                                        endpoint='http://server.test:5000/v3')
        mock_ks.return_value.authenticate.assert_called_once_with()

    def test_init_v3_bad_nocreds(self, mock_ks):
        """Test creating the client, no credentials."""
        self.ctx.auth_token = None
        self.ctx.trust_id = None
        self.ctx.username = None
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        self.assertRaises(exception.AuthorizationFailure,
                          solum_ks_client._v3_client_init)

    def test_init_trust_token_access(self, mock_ks):
        """Test creating the client, token auth."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.project_id = 'abcd1234'
        self.ctx.trust_id = None
        self.ctx.auth_token_info = {'access': {'token': {'id': 'placeholder'}}}

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        solum_ks_client.client
        self.assertIsNotNone(solum_ks_client._client)
        mock_ks.assert_called_once_with(auth_ref={'version': 'v2.0',
                                                  'token': {
                                                      'id': 'abcd1234'}},
                                        endpoint='http://server.test:5000/v3',
                                        auth_url='http://server.test:5000/v3')

    def test_init_trust_token_token(self, mock_ks):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.project_id = None
        self.ctx.trust_id = None
        self.ctx.auth_token_info = {'token': {}}

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        solum_ks_client.client
        self.assertIsNotNone(solum_ks_client._client)
        mock_ks.assert_called_once_with(auth_ref={'auth_token': 'abcd1234',
                                                  'version': 'v3'},
                                        endpoint='http://server.test:5000/v3',
                                        auth_url='http://server.test:5000/v3')

    def test_init_trust_token_none(self, mock_ks):
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.project_id = None
        self.ctx.trust_id = None
        self.ctx.auth_token_info = {'not_this': 'urg'}

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        self.assertRaises(exception.AuthorizationFailure,
                          solum_ks_client._v3_client_init)

    def test_create_trust_context_trust_id(self, mock_ks):
        """Test create_trust_context with existing trust_id."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.trust_id = 'atrust123'

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        trust_context = solum_ks_client.create_trust_context()
        self.assertEqual(self.ctx.to_dict(), trust_context.to_dict())
        mock_ks.assert_called_once_with(username='solum',
                                        auth_url='http://server.test:5000/v3',
                                        password='verybadpass',
                                        endpoint='http://server.test:5000/v3',
                                        trust_id='atrust123')
        mock_ks.return_value.authenticate.assert_called_once_with()

    def test_create_trust_context_trust_create(self, mock_ks):
        """Test create_trust_context when creating a trust."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')

        class FakeTrust(object):
            id = 'atrust123'

        cfg.CONF.set_override('trusts_delegated_roles',
                              ['solum_assembly_update'])

        getter_mock = mock.PropertyMock(side_effect=['1234', '5678'])
        type(mock_ks.return_value.auth_ref).user_id = getter_mock

        mock_ks.return_value.auth_ref.project_id = '42'
        mock_ks.return_value.trusts.create.return_value = FakeTrust()
        self.ctx.trust_id = None

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        trust_context = solum_ks_client.create_trust_context()

        # admin_client and user client
        expected = [mock.call(username='solum',
                              project_name='service',
                              password='verybadpass',
                              auth_url='http://server.test:5000/v3',
                              endpoint='http://server.test:5000/v3'),
                    mock.call(token='abcd1234',
                              project_id='fake_project_id',
                              auth_url='http://server.test:5000/v3',
                              endpoint='http://server.test:5000/v3')]

        self.assertEqual(expected, mock_ks.call_args_list)
        self.assertEqual([mock.call(), mock.call()],
                         mock_ks.return_value.authenticate.call_args_list)

        # trust creation
        self.assertEqual('atrust123', trust_context.trust_id)
        mock_ks.return_value.trusts.create.assert_called_once_with(
            trustor_user='5678',
            trustee_user='1234',
            project='42',
            impersonation=True,
            role_names=['solum_assembly_update'])

    def test_init_admin_client_denied(self, mock_ks):
        """Test the admin_client property, auth failure path."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.username = None
        self.ctx.password = None
        self.ctx.trust_id = None
        mock_ks.return_value.authenticate.return_value = False

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)

        # Define wrapper for property or the property raises the exception
        # outside of the assertRaises which fails the test
        def get_admin_client():
            solum_ks_client.admin_client

        self.assertRaises(exception.AuthorizationFailure,
                          get_admin_client)

    def test_init_lp_admin_client_denied(self, mock_ks):
        """Test the get_lp_admin_client property, auth failure path."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.username = None
        self.ctx.password = None
        self.ctx.trust_id = None
        mock_ks.return_value.authenticate.return_value = False

        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)

        # Define wrapper for property or the property raises the exception
        # outside of the assertRaises which fails the test
        def get_lp_admin_client():
            solum_ks_client.lp_admin_client

        self.assertRaises(exception.AuthorizationFailure,
                          get_lp_admin_client)

    def test_init_with_no_context(self, mock_ks):
        """Init with no context."""
        mock_ks.return_value.authenticate.return_value = False
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(None)
        self.assertEqual(solum_ks_client.endpoint,
                         'http://server.test:5000/v3')

    def test_trust_init_fail(self, mock_ks):
        """Test consuming a trust when initializing, error scoping."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.username = None
        self.ctx.auth_token = None
        self.ctx.trust_id = 'atrust123'
        mock_ks.return_value.auth_ref.trust_scoped = False

        self.assertRaises(exception.AuthorizationFailure,
                          solum_keystoneclient.KeystoneClientV3, self.ctx)

    def test_trust_init_token(self, mock_ks):
        """Test trust_id takes precedence when token specified."""
        # TODO(zhurong): should unskip the test
        self.skipTest('Skipping this test for bug #1686560')
        self.ctx.username = None
        self.ctx.trust_id = 'atrust123'
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        self.assertIsNotNone(solum_ks_client._client)
        mock_ks.assert_called_once_with(username='solum',
                                        auth_url='http://server.test:5000/v3',
                                        password='verybadpass',
                                        endpoint='http://server.test:5000/v3',
                                        trust_id='atrust123')
        mock_ks.return_value.authenticate.assert_called_once_with()

    def test_delete_trust(self, mock_ks):
        """Test delete_trust when deleting trust."""
        mock_ks.return_value.trusts.delete.return_value = None
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        self.assertIsNone(solum_ks_client.delete_trust(trust_id='atrust123'))
        mock_ks.return_value.trusts.delete.assert_called_once_with('atrust123')

    def test_delete_trust_not_found(self, mock_ks):
        """Test delete_trust when trust already deleted."""
        mock_delete = mock_ks.return_value.trusts.delete
        mock_delete.side_effect = kc_exception.NotFound()
        solum_ks_client = solum_keystoneclient.KeystoneClientV3(self.ctx)
        self.assertIsNone(solum_ks_client.delete_trust(trust_id='atrust123'))



#!/usr/bin/env python
# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import sys

from oslo_config import cfg
from oslo_log import log as logging

from solum.common import context
from solum.worker import api


LOG = logging.getLogger(__name__)


if __name__ == '__main__':
    conf_files = ['--config-file=/etc/solum/solum.conf']
    cfg.CONF(conf_files, project='solum')
    message = ' '.join(sys.argv[1:])
    api.API(context=context.RequestContext()).echo(message)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""API for interfacing with Solum Conductor."""

from oslo_config import cfg

from solum.common.rpc import service


class API(service.API):
    def __init__(self, context=None):
        cfg.CONF.import_opt('topic', 'solum.conductor.config',
                            group='conductor')
        super(API, self).__init__(context,
                                  topic=cfg.CONF.conductor.topic)

    def build_job_update(self, build_id, status, description, created_image_id,
                         docker_image_name, assembly_id):
        self._cast('build_job_update', build_id=build_id, status=status,
                   description=description, created_image_id=created_image_id,
                   docker_image_name=docker_image_name,
                   assembly_id=assembly_id)

    def update_assembly(self, assembly_id, data):
        self._cast('update_assembly', assembly_id=assembly_id, data=data)

    def update_image(self, image_id, status, external_ref=None,
                     docker_image_name=None):
        self._cast('update_image', image_id=image_id, status=status,
                   external_ref=external_ref,
                   docker_image_name=docker_image_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Extension(api_types.Base):
    """The Extension resource represents Provider modifications.

    This may include additional protocol semantics, resource types,
    application lifecycle states, resource attributes, etc. Anything may be
    added, as long as it does not contradict the base functionality offered
    by Solum.
    """

    version = wtypes.text
    "Version of the extension."

    documentation = common_types.Uri
    "Documentation URI to the extension."

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/extensions/logstash',
                   name='logstash',
                   type='extension',
                   tags=['large'],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='This logstash extension provides a tool for '
                               'managing your application events and logs.',
                   version='2.13',
                   documentation='http://example.com/docs/ext/logstash')



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sa

from solum import objects
from solum.objects import component as abstract
from solum.objects.sqlalchemy import models as sql

from oslo_utils import uuidutils


class Component(sql.Base, abstract.Component):
    """Represent an component in sqlalchemy."""

    __tablename__ = 'component'
    __resource__ = 'components'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    uuid = sa.Column(sa.String(36))
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))
    name = sa.Column(sa.String(100))
    component_type = sa.Column(sa.String(100))
    description = sa.Column(sa.String(255))
    tags = sa.Column(sa.Text)
    assembly_id = sa.Column(sa.Integer, sa.ForeignKey('assembly.id'))
    parent_component_id = sa.Column(sa.Integer, sa.ForeignKey('component.id'))
    resource_uri = sa.Column(sa.String(1024))
    heat_stack_id = sa.Column(sa.String(36))

    @property
    def assembly_uuid(self):
        if self.assembly_id is None:
            return None
        return objects.registry.Assembly.get_by_id(None, self.assembly_id).uuid

    @assembly_uuid.setter
    def assembly_uuid(self, assembly_uuid):
        assembly = objects.registry.Assembly.get_by_uuid(None, assembly_uuid)
        self.assembly_id = assembly.id

    @property
    def _extra_keys(self):
        return ['assembly_uuid']

    @staticmethod
    def assign_and_create(ctxt, assem, name, type, description, resource_uri,
                          stack_id):
        """Helper function to make creating components easier."""
        comp = objects.registry.Component()
        comp.uuid = uuidutils.generate_uuid()
        comp.name = name
        comp.component_type = type
        comp.description = description
        comp.assembly_id = assem.id
        comp.user_id = ctxt.user
        comp.project_id = ctxt.project_id
        comp.resource_uri = resource_uri
        comp.heat_stack_id = stack_id
        comp.create(ctxt)
        return comp


class ComponentList(abstract.ComponentList):
    """Represent a list of components in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return ComponentList(sql.model_query(context, Component))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import operation
from solum.api.controllers.v1.datamodel import types as api_types
from solum.i18n import _


SENSOR_TYPE = wtypes.Enum(str, 'str', 'float', 'int')


class Sensor(api_types.Base):
    """A Sensor resource represents exactly one supported sensor.

    Sensor resources represent dynamic data about resources,
    such as metrics or state. Sensor resources are useful for exposing data
    that changes rapidly, or that may need to be fetched from a secondary
    system.
    """

    documentation = common_types.Uri
    "Documentation URI for the sensor."

    target_resource = common_types.Uri
    "Target resource URI to the sensor."

    sensor_type = SENSOR_TYPE
    "Sensor data type."

    timestamp = datetime.datetime
    "Timestamp for Sensor."

    operations = [operation.Operation]
    "Operations that belong to the sensor."

    def get_value(self):
        if self.sensor_type == 'int':
            if int(self._value) != float(self._value):
                raise ValueError(_('Value "%s" is not an integer.') %
                                 str(self._value))
            return int(self._value)
        elif self.sensor_type == 'float':
            return float(self._value)
        else:
            return str(self._value)

    def set_value(self, value):
        # Store the value as-is, because we don't know the order that the
        # value and sensor_type are written, so convert to the desired type
        # in get_value().
        self._value = value

    value = wtypes.wsproperty(str, get_value, set_value, mandatory=False)
    "Value of the sensor."

    def __init__(self, **kwds):
        self._value = '0'
        super(Sensor, self).__init__(**kwds)

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/sensors/hb',
                   name='hb',
                   type='sensor',
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   description='A heartbeat sensor',
                   documentation='http://example.com/docs/heartbeat/',
                   target_resource='http://example.com/instances/uuid',
                   sensor_type='str',
                   value='30',
                   timestamp=datetime.datetime.utcnow(),
                   operations=[])
